@@ -8,6 +8,7 @@ Minimal Lipschitz Extensions), p-Laplace equations, and basic calculus on graphs
 """
 
 import numpy as np
+import matplotlib.pyplot as plt
 from scipy import sparse
 from scipy import spatial
 import scipy.sparse.linalg as splinalg
@@ -20,7 +21,7 @@ from . import utils
 
 class graph:
 
-    def __init__(self, W):
+    def __init__(self, W, labels=None, features=None, label_names=None, node_names=None):
         """Graph class
         ========
 
@@ -31,18 +32,29 @@ class graph:
         ----------
         W : (n,n) numpy array, matrix, or scipy sparse matrix
             Weight matrix representing the graph.
+        labels : (n,) numpy array (optional)
+            Node labels.
+        features : (n,k) numpy array (optional)
+            Node features.
+        label_names : list (optional)
+            Names corresponding to each label.
+        node_names : list (optional)
+            Names for each node in the graph.
         """
 
         self.weight_matrix = sparse.csr_matrix(W)
+        self.labels = labels
+        self.features = features
         self.num_nodes = W.shape[0]
+        self.label_names = label_names
+        self.node_names = node_names
 
         #Coordinates of sparse matrix for passing to C code
-        I,J,V = sparse.find(W)
-        self.I = I
-        self.J = J
-        self.V = V
-        K = np.array((J[1:] - J[:-1]).nonzero()) + 1
-        self.K = np.append(0,np.append(K,len(J)))
+        I,J,V = sparse.find(self.weight_matrix)
+        ind = np.argsort(I)
+        self.I,self.J,self.V = I[ind], J[ind], V[ind]
+        self.K = np.array((self.I[1:] - self.I[:-1]).nonzero()) + 1
+        self.K = np.append(0,np.append(self.K,len(self.I)))
 
         #For passing to C code
         self.I = np.ascontiguousarray(self.I, dtype=np.int32)
@@ -102,6 +114,62 @@ class graph:
         d = self.weight_matrix*np.ones(self.num_nodes)
         return d
 
+    def fiedler_vector(self, return_value=False, tol=1e-8):
+        """Fiedler Vector
+        ======
+
+        Computes the Fiedler vector for graph, which is the eigenvector 
+        of the graph Laplacian correpsonding to the second smallest eigenvalue.
+
+        Parameters
+        ----------
+        return_value : bool (optional), default=False
+            Whether to return Fiedler value.
+        tol : float (optional), default=0
+            Tolerance for eigensolvers.
+
+        Returns
+        -------
+        v : numpy array, float
+            Fiedler vector
+        l : float (optional)
+            Fiedler value
+        """
+        
+        #vals, vecs = self.eigen_decomp(k=2,method=method,tol=tol)
+        #if return_value:
+        #    return vecs[:,1], vals[1]
+        #else:
+        #    return vecs[:,1]
+
+        L = self.laplacian()
+        m = self.num_nodes
+        v = np.random.rand(m,1)
+        o = np.ones((m,1))/m
+        v -= np.sum(v)*o
+        d = self.degree_vector()
+        lam = 2*np.max(d)
+        M = lam*sparse.identity(m) - L
+        fval_old = v.T@(L@v)
+        err = 1
+        while err > tol:
+            x = M@v
+            x -= np.sum(x)*o
+            v = x/np.linalg.norm(x)
+            fval = v.T@(L@v)
+            err = abs(fval_old-fval)
+            fval_old = fval
+
+        v = v.flatten()
+        #Fix consistent sign
+        if v[0] > 0:
+            v = -v
+        if return_value:
+            return v, fval
+        else:
+            return v
+
+
 
     def degree_matrix(self, p=1):
         """Degree Matrix
@@ -492,7 +560,7 @@ class graph:
         normalization : {'combinatorial','randomwalk','normalized'}, default='combinatorial'
             Type of normalization of graph Laplacian to apply.
         method : {'exact','lowrank'}, default='exact'
-            Method for computing eigenvectors. 'exact' uses scipy.sparse.linalg.eigs, while
+            Method for computing eigenvectors. 'exact' uses scipy.sparse.linalg.svds, while
             'lowrank' uses a low rank approximation via randomized SVD. Lowrank is not 
             implemented for gamma > 0.
         k : int (optional), default=10
@@ -663,35 +731,8 @@ class graph:
 
             return vals, vecs
 
-    def fiedler_vector(self, method='exact', tol=0):
-        """Fiedler vector
-        ======
-
-        Computes the Fiedler vector, which is the second eigenvector for the 
-        combinatorial graph Laplacian \\(L = D-W\\).
-
-        Parameters
-        ----------
-        method : {'exact','lowrank'}, default='exact'
-            Method for computing eigenvectors. 'exact' uses scipy.sparse.linalg.eigs, while
-            'lowrank' uses a low rank approximation via randomized SVD.
-        tol : float (optional), default=0
-            tolerance for eigensolvers.
-
-
-        Returns
-        -------
-        fiedler_vector : numpy array, float 
-            Contents of fiedler vector.
-        """
-
-        vals, vecs = self.eigen_decomp(normalization='combinatorial', method=method, k=2, tol=tol)
-        fiedler_vector = vecs[:,1]
-
-        return fiedler_vector
-
-    def peikonal(self, bdy_set, bdy_val=0, f=1, p=1, u0=None, solver='fmm', max_num_it=1e5, 
-                                                     tol=1e-3, num_bisection_it=30, prog=False,):
+    def peikonal(self, bdy_set, bdy_val=0, f=1, p=1, nl_bdy=False, u0=None, solver='fmm',
+                              max_num_it=1e5, tol=1e-3, num_bisection_it=30, prog=False,):
         """p-eikonal equation 
         =====================
 
@@ -711,6 +752,8 @@ class graph:
             is interpreted as a constant vector of the graph.
         p : float (optional), default=1
             Value of exponent p in the p-eikonal equation.
+        nl_bdy : bool (optional), default = False
+            Whether to extend the boundary conditions to non-local ones (to graph neighbors).
         solver : {'fmm', 'gauss-seidel'}, default='fmm'
             Solver for p-eikonal equation.
         u0 : numpy array (float, optional), default=None
@@ -770,6 +813,18 @@ class graph:
 
         #Convert boundary data to standard format
         bdy_set, bdy_val = utils._boundary_handling(bdy_set, bdy_val)
+        
+        #Extend boundary data if nl_bdy=True
+        if nl_bdy:
+            D = self.degree_matrix(p=-1)
+            bdy_mask = np.zeros(n)
+            bdy_mask[bdy_set] = 1
+            bdy_dilate = (D*self.weight_matrix*bdy_mask) > 0
+            bdy_set = bdy_set = np.where(bdy_dilate)[0]
+            bdy_val_all = np.zeros(n)
+            bdy_val_all[bdy_mask==1] = bdy_val
+            bdy_val = D*self.weight_matrix*bdy_val_all
+            bdy_val = bdy_val[bdy_set]
 
         #Type casting and memory blocking
         u = np.ascontiguousarray(u,dtype=np.float64)
@@ -778,9 +833,9 @@ class graph:
         bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
 
         if solver == 'fmm':
-            cextensions.peikonal_fmm(u,self.I,self.K,self.V,bdy_set,f,bdy_val,p,num_bisection_it)
+            cextensions.peikonal_fmm(u,self.J,self.K,self.V,bdy_set,f,bdy_val,p,num_bisection_it)
         else:
-            cextensions.peikonal(u,self.I,self.K,self.V,bdy_set,f,bdy_val,p,max_num_it,tol,num_bisection_it,prog)
+            cextensions.peikonal(u,self.J,self.K,self.V,bdy_set,f,bdy_val,p,max_num_it,tol,num_bisection_it,prog)
 
         return u
 
@@ -860,7 +915,7 @@ class graph:
         bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
         f = np.ascontiguousarray(f,dtype=np.float64)
 
-        cextensions.dijkstra_hl(dist_func,cp,self.I,self.K,self.V,bdy_set,bdy_val,f,1.0,max_dist)
+        cextensions.dijkstra_hl(dist_func,cp,self.J,self.K,self.V,bdy_set,bdy_val,f,1.0,max_dist)
 
         if return_cp:
             return dist_func, cp
@@ -955,14 +1010,14 @@ class graph:
         bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
         f = np.ascontiguousarray(f,dtype=np.float64)
 
-        cextensions.dijkstra(dist_func,cp,self.I,self.K,self.V,bdy_set,bdy_val,f,1.0,max_dist)
+        cextensions.dijkstra(dist_func,cp,self.J,self.K,self.V,bdy_set,bdy_val,f,1.0,max_dist)
 
         if return_cp:
             return dist_func, cp
         else:
             return dist_func
 
-    def plaplace(self, bdy_set, bdy_val, p, tol=1e-1, max_num_it=1e6, prog=False):
+    def plaplace(self, bdy_set, bdy_val, p, tol=1e-1, max_num_it=1e6, prog=False, fast=True):
         """Game-theoretic p-Laplacian
         ======
 
@@ -989,6 +1044,9 @@ class graph:
             Maximum number of iterations.
         prog : bool (optional), default=False
             Toggles whether to print progress information.
+        fast : bool (optional), default=True
+            Whether to use constant \\(w_{ij}=1\\) weights for the infinity-Laplacian
+            which allows a faster algorithm to be used.
 
         Returns
         -------
@@ -1025,25 +1083,41 @@ class graph:
         from . import cextensions
         
         n = self.num_nodes
+        alpha = 1/(p-1)
+        beta = 1-alpha
 
         #Convert boundary data to standard format
         bdy_set, bdy_val = utils._boundary_handling(bdy_set, bdy_val)
 
-        uu = np.max(bdy_val)*np.ones((n,))
-        ul = np.min(bdy_val)*np.ones((n,))
+        #If fast solver
+        if fast:
 
-        #Set labels
-        uu[bdy_set] = bdy_val
-        ul[bdy_set] = bdy_val
+            u = np.zeros((n,))        #Initial condition
 
-        #Type casting and memory blocking
-        uu = np.ascontiguousarray(uu,dtype=np.float64)
-        ul = np.ascontiguousarray(ul,dtype=np.float64)
-        bdy_set = np.ascontiguousarray(bdy_set,dtype=np.int32)
-        bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
+            #Type casting and memory blocking
+            u = np.ascontiguousarray(u,dtype=np.float64)
+            bdy_set = np.ascontiguousarray(bdy_set,dtype=np.int32)
+            bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
 
-        cextensions.lp_iterate(uu,ul,self.I,self.J,self.V,bdy_set,bdy_val,p,float(max_num_it),float(tol),float(prog))
-        u = (uu+ul)/2
+            weighted = False
+            tol = 1e-6
+            cextensions.lip_iterate(u,self.J,self.I,self.V,bdy_set,bdy_val,max_num_it,tol,float(prog),float(weighted),float(alpha),float(beta))
+        else:
+            uu = np.max(bdy_val)*np.ones((n,))
+            ul = np.min(bdy_val)*np.ones((n,))
+
+            #Set labels
+            uu[bdy_set] = bdy_val
+            ul[bdy_set] = bdy_val
+
+            #Type casting and memory blocking
+            uu = np.ascontiguousarray(uu,dtype=np.float64)
+            ul = np.ascontiguousarray(ul,dtype=np.float64)
+            bdy_set = np.ascontiguousarray(bdy_set,dtype=np.int32)
+            bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
+
+            cextensions.lp_iterate(uu,ul,self.J,self.I,self.V,bdy_set,bdy_val,p,float(max_num_it),float(tol),float(prog))
+            u = (uu+ul)/2
 
         return u
 
@@ -1083,16 +1157,20 @@ class graph:
 
         #Variables
         n = self.num_nodes
-        k = len(bdy_set)
         u = np.zeros((n,))        #Initial condition
         max_num_it = float(max_num_it)
+        alpha = 0
+        beta = 1
+
+        #Convert boundary data to standard format
+        bdy_set, bdy_val = utils._boundary_handling(bdy_set, bdy_val)
 
         #Type casting and memory blocking
         u = np.ascontiguousarray(u,dtype=np.float64)
         bdy_set = np.ascontiguousarray(bdy_set,dtype=np.int32)
         bdy_val = np.ascontiguousarray(bdy_val,dtype=np.float64)
 
-        cextensions.lip_iterate(u,self.I,self.J,self.V,bdy_set,bdy_val,max_num_it,tol,float(prog),float(weighted))
+        cextensions.lip_iterate(u,self.J,self.I,self.V,bdy_set,bdy_val,max_num_it,tol,float(prog),float(weighted),float(alpha),float(beta))
 
         return u
 
@@ -1176,5 +1254,78 @@ class graph:
 
         return u
 
+    def draw(self,X=None,c=None,cmap='viridis',markersize=None,linewidth=None,edges=True):
+        """Draw Graph
+        ======
+
+        Draws a planar representation of a graph using metric MDS. 
+
+        Parameters
+        ----------
+        X : (n,2) numpy array (optional)
+            Coordinates of graph vertices to draw. If not provided, uses metric MDS.
+        c : (n,) numpy array (optional)
+            Colors of vertices. If not provided, vertices are colored black.
+        cmap : string (optional)
+            Colormap. Default is 'viridis'.
+        markersize : float (optional)
+            Markersize.
+        linewidth : float (optional)
+            Linewidth.
+        edges : bool (optional)
+            Whether to plot edges (default=True)
+        """
+
+        plt.figure()
+        n = self.num_nodes
+
+        #If points are not provided, we use metric MDS
+        if X is None:
+            J = np.eye(n) - (1/n)*np.ones((n,n))
+            dist = np.zeros((n,n))
+            for i in range(n):
+                dist[i,:] = self.dijkstra([i])
+            H = -(1/2)*J@dist@J
+
+            #Need to sort eigenvalues, since H may not be positive semidef
+            vals,V = sparse.linalg.eigsh(H,k=10,which='LM')
+            ind = np.argsort(-vals)
+            V = V[:,ind]
+            vals = vals[ind]
+
+            #Get top eigenvectors and square roots of positive parts of eigenvalues
+            P = V[:,:2]
+            S = np.maximum(vals[:2],0)**(1/2)
+
+            #MDS embedding
+            X = P@np.diag(S)
+
+        #Plot points
+        x,y = X[:,0],X[:,1]
+        if c is None:
+            if markersize is None:
+                plt.scatter(x,y,zorder=2)
+            else:
+                plt.scatter(x,y,s=markersize,zorder=2)
+        else:
+            if markersize is None:
+                plt.scatter(x,y,c=c,cmap=cmap,zorder=2)
+            else:
+                plt.scatter(x,y,c=c,cmap=cmap,s=markersize,zorder=2)
+
+        #Draw edges
+        if edges:
+            for i in range(n):
+                nn = self.weight_matrix[i,:].nonzero()[1]
+                for j in nn:
+                    if linewidth is None:
+                        plt.plot([x[i],x[j]],[y[i],y[j]],c='black',zorder=0)
+                    else:
+                        plt.plot([x[i],x[j]],[y[i],y[j]],c='black',linewidth=linewidth,zorder=0)
+
+
+
+
+
 
 
