@@ -25,7 +25,7 @@ train_labels = labels[train_ind]
 model = gl.ssl.laplace(W)
 pred_labels = model.fit_predict(train_ind, train_labels)
 
-accuracy = gl.ssl.ssl_accuracy(pred_labels, labels, len(train_ind))   
+accuracy = gl.ssl.ssl_accuracy(pred_labels, labels, train_ind)   
 print("Accuracy: %.2f%%"%accuracy)
 
 plt.scatter(X[:,0],X[:,1], c=pred_labels)
@@ -49,7 +49,7 @@ models = [gl.ssl.laplace(W), gl.ssl.poisson(W)]
 
 for model in models:
     pred_labels = model.fit_predict(train_ind,train_labels)
-    accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,len(train_ind))
+    accuracy = gl.ssl.ssl_accuracy(pred_labels, labels, train_ind)
     print(model.name + ': %.2f%%'%accuracy)
 ```
 Comparing different methods
@@ -103,11 +103,11 @@ model = gl.ssl.laplace(W, class_priors=class_priors)
 model.fit(train_ind,train_labels)
 
 pred_labels = model.predict(ignore_class_priors=True)
-accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,len(train_ind))
+accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,train_ind)
 print(model.name + ' without class priors: %.2f%%'%accuracy)
 
 pred_labels = model.predict()
-accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,len(train_ind))
+accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,train_ind)
 print(model.name + ' with class priors: %.2f%%'%accuracy)
 ```
 """
@@ -364,12 +364,12 @@ class ssl:
             pred_labels = self.fit_predict(train_ind, train_labels)
 
             #Compute accuracy
-            accuracy = ssl_accuracy(pred_labels,labels,num_train)
+            accuracy = ssl_accuracy(pred_labels,labels,train_ind)
 
             #If class priors were provided, check accuracy without priors
             if self.class_priors is not None:
                 pred_labels = self.predict(ignore_class_priors=True)
-                accuracy_without_priors = ssl_accuracy(pred_labels,labels,num_train)
+                accuracy_without_priors = ssl_accuracy(pred_labels,labels,train_ind)
             
             #Print to terminal
             if self.class_priors is None:
@@ -566,7 +566,7 @@ class poisson(ssl):
         model = gl.ssl.poisson(W, solver='gradient_descent')
         pred_labels = model.fit_predict(train_ind, train_labels)
 
-        accuracy = gl.ssl.ssl_accuracy(pred_labels, labels, len(train_ind))   
+        accuracy = gl.ssl.ssl_accuracy(pred_labels, labels, train_ind)   
         print("Accuracy: %.2f%%"%accuracy)
 
         plt.scatter(X[:,0],X[:,1], c=pred_labels)
@@ -673,7 +673,7 @@ class poisson(ssl):
                     if all_labels is not None:
                         self.prob = u
                         labels = self.predict()
-                        acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                        acc = ssl_accuracy(labels,all_labels,train_ind)
                         print('%d,Accuracy = %.2f'%(T,acc))
                 
         #Use spectral solver
@@ -741,7 +741,7 @@ class poisson_mbo(ssl):
         model = gl.ssl.poisson_mbo(W, class_priors)
         pred_labels = model.fit_predict(train_ind,train_labels,all_labels=labels)
 
-        accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,len(train_ind))
+        accuracy = gl.ssl.ssl_accuracy(labels,pred_labels,train_ind)
         print(model.name + ': %.2f%%'%accuracy)
         ```
         Reference
@@ -833,7 +833,7 @@ class poisson_mbo(ssl):
 
             #Compute accuracy if all labels are provided
             if all_labels is not None:
-                acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                acc = ssl_accuracy(labels,all_labels,train_ind)
                 print('%d, Accuracy = %.2f'%(i,acc))
 
         return u
@@ -999,7 +999,7 @@ class multiclass_mbo(ssl):
             if all_labels is not None:
                 self.prob = u.T
                 labels = self.predict()
-                acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                acc = ssl_accuracy(labels,all_labels,train_ind)
                 print('Accuracy = %.2f'%acc)
 
         return u.T
@@ -1097,7 +1097,7 @@ class modularity_mbo(ssl):
             if all_labels is not None:
                 self.prob = u
                 labels = self.predict()
-                acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                acc = ssl_accuracy(labels,all_labels,train_ind)
                 print('Accuracy = %.2f'%acc)
 
         return u
@@ -1105,15 +1105,15 @@ class modularity_mbo(ssl):
 
 class laplace(ssl):
     def __init__(self, W=None, class_priors=None, X=None, reweighting='none', normalization='combinatorial', 
-                 tau=0, mean_shift=False, tol=1e-5, alpha=2, zeta=1e7, r=0.1):
+                 tau=0, order=1, mean_shift=False, tol=1e-5, alpha=2, zeta=1e7, r=0.1):
         """Laplace Learning
         ===================
 
         Semi-supervised learning via the solution of the Laplace equation
-        \\[\\tau u_j + L u_j = 0, \\ \\ j \\geq m+1,\\]
+        \\[\\tau u_j + L^m u_j = 0, \\ \\ j \\geq m+1,\\]
         subject to \\(u_j = y_j\\) for \\(j=1,\\dots,m\\), where \\(L=D-W\\) is the 
         combinatorial graph Laplacian and \\(y_j\\) for \\(j=1,\\dots,m\\) are the 
-        label vectors. 
+        label vectors. Default order is m=1, and m > 1 corresponds to higher order Laplace Learning.
 
         The original method was introduced in [1]. This class also implements reweighting 
         schemes `poisson` proposed in [2], `wnll` proposed in [3], and `properly`, proposed in [4].
@@ -1137,6 +1137,8 @@ class laplace(ssl):
             must supply the data features `X`.
         tau : float or numpy array (optional), default=0
             Zeroth order term in Laplace equation. Can be a scalar or vector.
+        order : integer (optional), default=1
+            Power m for higher order Laplace learning. Currently only integers are allowed. 
         mean_shift : bool (optional), default=False
             Whether to shift output to mean zero.
         tol : float (optional), default=1e-5
@@ -1169,6 +1171,7 @@ class laplace(ssl):
         self.normalization = normalization
         self.mean_shift = mean_shift
         self.tol = tol
+        self.order = order
         self.X = X
 
         #Set up tau
@@ -1189,6 +1192,8 @@ class laplace(ssl):
         if self.mean_shift:
             fname += '_meanshift'
             self.name += ' with meanshift'
+        if self.order > 1:
+            self.name += ' order %d'%int(self.order)
         if np.max(self.tau) > 0:
             fname += '_tau_%.3f'%np.max(self.tau)
             self.name += ' tau=%.3f'%np.max(self.tau)
@@ -1213,6 +1218,12 @@ class laplace(ssl):
         
         #tau + Graph Laplacian and one-hot labels
         L = sparse.spdiags(self.tau, 0, G.num_nodes, G.num_nodes) + G.laplacian(normalization=self.normalization)
+        if self.order > 1:
+            Lpow = L*L
+            if self.order > 2:
+                for i in range(2,self.order):
+                    Lpow = L*Lpow
+            L = Lpow
         F = utils.labels_to_onehot(train_labels)
 
         #Locations of unlabeled points
@@ -1322,7 +1333,7 @@ class dynamic_label_propagation(ssl):
                 if all_labels is not None:
                     self.prob = np.array(u)
                     labels = self.predict()
-                    acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                    acc = ssl_accuracy(labels,all_labels,train_ind)
                     print('Accuracy = %.2f'%acc)
 
             u = np.array(u)
@@ -1407,7 +1418,7 @@ class centered_kernel(ssl):
             if all_labels is not None:
                 self.prob = u
                 labels = self.predict()
-                acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                acc = ssl_accuracy(labels,all_labels,train_ind)
                 print('Accuracy = %.2f'%acc)
             
         return u
@@ -1489,7 +1500,7 @@ class sparse_label_propagation(ssl):
             if all_labels is not None:
                 self.prob = u.T
                 labels = self.predict()
-                acc = ssl_accuracy(labels,all_labels,len(train_ind))
+                acc = ssl_accuracy(labels,all_labels,train_ind)
                 print('%d,Accuracy = %.2f'%(i,acc))
  
         return u.T
@@ -1665,7 +1676,7 @@ class peikonal(ssl):
 
 
 class plaplace(ssl):
-    def __init__(self, W=None, class_priors=None, p=10, max_num_it=1e6, tol=1e-1):
+    def __init__(self, W=None, class_priors=None, p=10, max_num_it=1e6, tol=1e-1, fast=True):
         """Graph p-laplace classifier
         ===================
 
@@ -1685,6 +1696,9 @@ class plaplace(ssl):
             Maximum number of iterations
         tol : float (optional), default=1e-3
             Tolerance with which to solve the equation.
+        fast : bool (optional), default=True
+            Whether to use constant \\(w_{ij}=1\\) weights for the infinity-Laplacian
+            which allows a faster algorithm to be used.
 
         References
         ----------
@@ -1696,6 +1710,7 @@ class plaplace(ssl):
         self.max_num_it = max_num_it
         self.tol = tol
         self.onevsrest = True
+        self.fast = fast
 
         #Setup accuracy filename and model name
         self.accuracy_filename = '_plaplace_p%.2f'%self.p 
@@ -1703,7 +1718,7 @@ class plaplace(ssl):
 
 
     def _fit(self, train_ind, train_labels, all_labels=None):
-        u = self.graph.plaplace(train_ind, train_labels, self.p, max_num_it=self.max_num_it, tol=self.tol)
+        u = self.graph.plaplace(train_ind, train_labels, self.p, max_num_it=self.max_num_it, tol=self.tol,fast=self.fast)
         return u
 
 
@@ -1771,7 +1786,7 @@ class randomwalk(ssl):
 
         return u
 
-def ssl_accuracy(pred_labels, true_labels, num_train):   
+def ssl_accuracy(pred_labels, true_labels, train_ind):   
     """SSL Accuacy
     ======
 
@@ -1785,21 +1800,32 @@ def ssl_accuracy(pred_labels, true_labels, num_train):
         Predicted labels
     true_labels : numpy array, int
         True labels
-    num_train : int
-        Number of training points (removed from dataset during accuracy computation).
+    train_ind : numpy array, int 
+        Indices of training points, which will be removed from the accuracy computation.
 
     Returns
     -------
     accuracy : float
         Accuracy as a number in [0,100].
     """
+
+        #Remove labeled data
+    mask = np.ones(len(pred_labels),dtype=bool)
+    if type(train_ind) != np.ndarray:
+        print("Warning: ssl_accuracy has been updated and now requires the user to provide the indices of the labeled points, and not just the number of labels. Accuracy computation will be incorrect unless you update your code accordingly. See https://jwcalder.github.io/GraphLearning/ssl.html#two-moons-example")
+    else:
+        mask[train_ind]=False
+
+    pred_labels = pred_labels[mask]
+    true_labels = true_labels[mask]
+
     #Remove unlabeled nodes
     I = true_labels >=0
     pred_labels = pred_labels[I]
     true_labels = true_labels[I]
 
     #Compute accuracy
-    return 100*np.maximum(np.sum(pred_labels==true_labels)-num_train,0)/(len(true_labels)-num_train)
+    return 100*np.mean(pred_labels==true_labels)
 
 
 def accuracy_plot(model_list, tag='', testerror=False, savefile=None, title=None, errorbars=False, 
