@@ -15,6 +15,55 @@ from . import utils
 #Directory to store knn data
 knn_dir = os.path.abspath(os.path.join(os.getcwd(),'knn_data'))
 
+def grid_graph(n,m=None,return_xy=False):
+    '''Grid graph
+    ========
+
+    Returns the adjacency matrix for a graph on a regular grid.
+
+    Parameters
+    ----------
+    n : int
+        Number of pixels wide. Or if n is an image, then (n,m) are taken as the shape
+        of the first and second dimensions of the image.
+    m : int (Optional)
+        Number of pixels high. Does not need to be specified if n is an image.
+    return_xy : bool (optional, default=False)
+        Whether to return x,y coordinates as well
+
+    Returns
+    -------
+    W : (m*n,m*n) scipy sparse matrix
+        Weight matrix of nearest neighbor graph on (m,n) grid
+    X : (m*n,2) numpy array, float
+        Coordiantes of vertices of grid
+    '''
+
+    if m is None:
+        s = n.shape
+        m = s[1]
+        n = s[0]
+        
+    xm, ym = np.meshgrid(np.arange(m),np.arange(n))
+    c = (xm + m*ym).flatten()
+    ne = (np.clip(xm + 1,0,m-1) + m*ym).flatten()
+    nw = (np.clip(xm - 1,0,m-1) + m*ym).flatten()
+    nn = (xm + m*np.clip(ym + 1,0,n-1)).flatten()
+    ns = (xm + m*np.clip(ym - 1,0,n-1)).flatten()
+    edges = np.vstack((c,ne)).T
+    edges = np.vstack((edges,np.vstack((c,nw)).T))
+    edges = np.vstack((edges,np.vstack((c,nn)).T))
+    edges = np.vstack((edges,np.vstack((c,ns)).T))
+    ind = edges[:,0] != edges[:,1]
+    edges = edges[ind,:]
+    W = sparse.coo_matrix((np.ones(len(edges)), (edges[:,0],edges[:,1])),shape=(m*n,m*n))
+
+    if return_xy:
+        X = np.vstack((xm.flatten(),ym.flatten())).T
+        return W.tocsr(),X.astype(float)
+    else:
+        return W.tocsr()
+
 def knn(data, k, kernel='gaussian', eta=None, symmetrize=True, metric='raw', similarity='euclidean', knn_data=None):
     """knn weight matrix
     ======
@@ -27,12 +76,14 @@ def knn(data, k, kernel='gaussian', eta=None, symmetrize=True, metric='raw', sim
         If numpy array, n data points, each of dimension m, if string, then 'mnist', 'fashionmnist', or 'cifar'
     k : int
         Number of nearest neighbors to use.
-    kernel : string (optional), {'uniform','gaussian','singular','distance'}, default='gaussian'
+    kernel : string (optional), {'uniform','gaussian','symgaussian','singular','distance'}, default='gaussian'
         The choice of kernel in computing the weights between \\(x_i\\) and each of its k 
         nearest neighbors. We let \\(d_k(x_i)\\) denote the distance from \\(x_i\\) to its kth 
         nearest neighbor. The choice 'uniform' corresponds to \\(w_{i,j}=1\\) and constitutes
         an unweighted k nearest neighbor graph, 'gaussian' corresponds to
         \\[ w_{i,j} = \\exp\\left(\\frac{-4\\|x_i - x_j\\|^2}{d_k(x_i)^2} \\right), \\]
+        'symgaussian' corresponds to
+        \\[ w_{i,j} = \\exp\\left(\\frac{-4\\|x_i - x_j\\|^2}{d_k(x_i)d_k(x_j)} \\right), \\]
         'distance' corresponds to
         \\[ w_{i,j} = \\|x_i - x_j\\|, \\]
         and 'singular' corresponds to 
@@ -45,8 +96,10 @@ def knn(data, k, kernel='gaussian', eta=None, symmetrize=True, metric='raw', sim
     symmetrize : bool (optional), default=True, except when kernel='singular'
         Whether or not to symmetrize the weight matrix before returning. Symmetrization is 
         performed by returning \\( (W + W^T)/2 \\), except for when kernel='distance, in 
-        which case the symmetrized edge weights are the true distances. Default for symmetrization
-        is True, unless the kernel is 'singular', in which case it is False.
+        which case the symmetrized edge weights are the true distances, kernel='uniform', 
+        where the weights are all 0/1, or kernel='symgaussian', where the same formula 
+        is used for symmetry. Default for symmetrization is True, unless the kernel is
+        'singular', in which case it is False.
     metric : string (optional), default='raw'
         Metric identifier if data is a string (i.e., a dataset).
     similarity : {'euclidean','angular','manhattan','hamming','dot'} (optional), default='euclidean'
@@ -88,6 +141,9 @@ def knn(data, k, kernel='gaussian', eta=None, symmetrize=True, metric='raw', sim
             D = knn_dist*knn_dist
             eps = D[:,k-1]
             weights = np.exp(-4*D/eps[:,None])
+        elif kernel == 'symgaussian':
+            eps = knn_dist[:,k-1]
+            weights = np.exp(-4 * knn_dist * knn_dist / eps[:,None] / eps[knn_ind])
         elif kernel == 'distance':
             weights = knn_dist
         elif kernel == 'singular':
@@ -118,9 +174,12 @@ def knn(data, k, kernel='gaussian', eta=None, symmetrize=True, metric='raw', sim
     if symmetrize:
         if kernel in ['distance','uniform']:
             W = utils.sparse_max(W, W.transpose())
+        elif kernel == 'symgaussian':
+            W = W + W.T.multiply(W.T > W) - W.multiply(W.T > W)
         else:
             W = (W + W.transpose())/2;
 
+
     return W
 
 def epsilon_ball(data, epsilon, kernel='gaussian', features=None, epsilon_f=1, eta=None, zero_diagonal=False):
@@ -426,8 +485,8 @@ def vae(data, layer_widths=[400,20], no_cuda=False, batch_size=128, epochs=100,
     pred_labels_raw = gl.ssl.poisson(W_raw).fit_predict(train_ind,train_labels)
     pred_labels_vae = gl.ssl.poisson(W_vae).fit_predict(train_ind,train_labels)
 
-    accuracy_raw = gl.ssl.ssl_accuracy(labels,pred_labels_raw,len(train_ind))
-    accuracy_vae = gl.ssl.ssl_accuracy(labels,pred_labels_vae,len(train_ind))
+    accuracy_raw = gl.ssl.ssl_accuracy(labels,pred_labels_raw,train_ind)
+    accuracy_vae = gl.ssl.ssl_accuracy(labels,pred_labels_vae,train_ind)
 
     print('Raw Accuracy: %.2f%%'%accuracy_raw)
     print('VAE Accuracy: %.2f%%'%accuracy_vae)
